from django.http import HttpResponse
from django.http import JsonResponse
from django.views import View
from django.core.exceptions import ObjectDoesNotExist
from .util import util

from .models import Clients


def _validate_token(request):
    if not 'Token' in request.headers:
        return False

    token = request.headers['Token']
    result = util.validate_token(token)
    return result


class ViewClients(View):

    def get(self, request):
        if not _validate_token(request):
            return HttpResponse(status=401)

        rfc = request.GET['rfc']
        try:
            obj = Clients.objects.filter(rfc=rfc).values()[0]
        except IndexError:
            msg = {'error': 'Cliente no existe'}
            return JsonResponse(msg, safe=False, status=202)

        del obj['key']
        del obj['cer']
        return JsonResponse(obj, safe=False)

    def post(self, request):
        if not _validate_token(request):
            return HttpResponse(status=401)

        post = request.POST
        files = request.FILES

        result, data = util.validate_client(post, files)
        if not result:
            return HttpResponse(data, status=202)

        if Clients.objects.filter(rfc=data['rfc']).exists():
            msg = 'Cliente ya existe'
            return HttpResponse(msg, status=202)

        obj = Clients.objects.create(**data)

        return HttpResponse(status=201)

    def delete(self, request, *args, **kwargs):
        if not _validate_token(request):
            return HttpResponse(status=401)

        rfc = request.GET['rfc']
        try:
            obj = Clients.objects.get(rfc=rfc)
        except ObjectDoesNotExist:
            msg = 'Cliente no existe'
            return HttpResponse(msg, status=202)

        obj.delete()
        return HttpResponse()


class ViewCfdi(View):

    def post(self, request):
        if not _validate_token(request):
            return HttpResponse(status=401)

        result, data = util.validate_cfdi(request.body)
        if not result:
            return HttpResponse(data, status=202)

        xml, msg = util.send_stamp(data)
        if msg:
            return HttpResponse(msg, status=202)

        return JsonResponse(xml, status=201)
